@@ -49,7 +49,7 @@
 #' }
 #'
 #' @import torch ggplot2 purrr
-#' @importFrom stats  quantile runif dunif punif qunif bw.nrd0 bw.nrd dnorm approxfun fft median sd rt coef pnorm rnorm uniroot
+#' @importFrom stats  quantile runif dunif punif qunif bw.nrd0 bw.nrd dnorm approxfun fft median sd rt coef pnorm rnorm splinefun kmeans
 #' @importFrom imputeTS  na_kalman
 #' @importFrom scales  number
 #' @importFrom lubridate  seconds_to_period period
@@ -97,7 +97,10 @@ temper <- function(ts, future, past, latent_dim, n_trees = 30, depth = 6,
   y_hat <- model(new_x)
   raw_preds <- as.matrix(torch_stack(y_hat$pred)$squeeze())
   proj_space <- t(apply(raw_preds, 2, function(x) tail(ts, 1) * cumprod(1 + x)))
-  pred_funs <- apply(proj_space, 2, function(x) gmix(x, K.max = n_bases, seed = seed))
+
+  if(future == 1){proj_space <- matrix(proj_space, ncol = 1)}
+
+  pred_funs <- apply(proj_space, 2, function(x) gmix(x, K_max = n_bases, seed = seed))
   names(pred_funs) <- paste0("t", 1:future)
 
   plot <- plot_graph(ts, pred_funs, alpha = alpha, dates = dates)
@@ -277,6 +280,7 @@ plot_graph <- function(ts, pred_funs, alpha = 0.05, dates = NULL, line_size = 1.
                        label_x = "Horizon", label_y= "Forecasted Var", date_format = "%b-%Y")
 {
   preds <- Reduce(rbind, map(pred_funs, ~ quantile(.x$rfun(1000), probs = c(alpha, 0.5, (1-alpha)))))
+  if(length(pred_funs)==1){preds <- matrix(preds, nrow = 1)}
   colnames(preds) <- c("lower", "median", "upper")
   future <- nrow(preds)
 
@@ -326,42 +330,48 @@ smart_reframer <- function(ts, seq_len, stride)
 
 #' @keywords internal
 gmix <- function(x,
-                 K.max = 10,     # upper bound on clusters
+                 K_max = 10,     # upper bound on clusters
                  seed   = 1,     # k-means reproducibility
                  ...) {          # extra args to kmeans()
   stopifnot(is.numeric(x), is.vector(x))
   n <- length(x)
 
+  K_pos <- length(unique(x))
+  K_eff <- min(K_max, K_pos)
+
+  if (K_eff < K_max) warning("Reduced k from ", K_max, " to ", K_eff,
+                             " because of only ", K_pos, " distinct points.")
+
   ## -------------------------------------------------------------- ##
-  ## 1.  Compute WSS for k = 1 … K.max ----------------------------- ##
+  ## 1.  Compute WSS for k = 1 ... K_max ----------------------------- ##
   ## -------------------------------------------------------------- ##
-  wss <- numeric(K.max)
+  wss <- numeric(K_eff)
   set.seed(seed)
-  for (k in 1:K.max) {
-    wss[k] <- stats::kmeans(x, centers = k, ...)$tot.withinss
+  for (k in 1:K_eff) {
+    wss[k] <- kmeans(x, centers = k, ...)$tot.withinss
   }
 
   ## -------------------------------------------------------------- ##
   ## 2.  Detect the elbow (max curvature) ------------------------- ##
-  ## -------------------------------------------------------------- ##
-  # First & second finite differences: ΔWSS(k) = WSS(k-1) – WSS(k)
-  d1  <- -diff(wss)                  # length K.max-1
-  d2  <- diff(d1)                    # length K.max-2
+  ## ----------- --------------------------------------------------- ##
+  # First & second finite differences: deltaWSS(k) = WSS(k-1) - WSS(k)
+  d1  <- -diff(wss)                  # length K_eff-1
+  d2  <- diff(d1)                    # length K_eff-2
   k_hat <- which.max(d2) + 1L        # add 1 because d2 starts at k = 3
 
   ## Fallback if curvature is flat (rare): default to 2 clusters
   if (length(k_hat) == 0 || is.na(k_hat) || k_hat < 2) k_hat <- 2
 
   ## -------------------------------------------------------------- ##
-  ## 3.  Final k-means with k = k̂ -------------------------------- ##
+  ## 3.  Final k-means with k = k^ -------------------------------- ##
   ## -------------------------------------------------------------- ##
   set.seed(seed)
-  km <- stats::kmeans(x, centers = k_hat, ...)
+  km <- kmeans(x, centers = k_hat, ...)
 
   clusters <- km$cluster
   centers  <- as.numeric(km$centers)
   sizes    <- km$size
-  weights  <- sizes / n                           # π_g
+  weights  <- sizes / n
 
   # Component SDs (protect 1-point clusters with a small positive value)
   sigmas <- vapply(split(x, clusters), function(v)
@@ -371,16 +381,41 @@ gmix <- function(x,
   ## 4.  Mixture helpers ------------------------------------------ ##
   ## -------------------------------------------------------------- ##
   pdf_fun <- function(z)
-    vapply(z, function(zz) sum(weights * dnorm(zz, centers, sigmas)), numeric(1))
+    vapply(z, \(zz) sum(weights * dnorm(zz, centers, sigmas)), numeric(1))
 
   cdf_fun <- function(z)
-    vapply(z, function(zz) sum(weights * pnorm(zz, centers, sigmas)), numeric(1))
+    vapply(z, \(zz) sum(weights * pnorm(zz, centers, sigmas)), numeric(1))
 
-  lower <- min(x) - 6 * max(sigmas)
-  upper <- max(x) + 6 * max(sigmas)
+  ## --- 1.  Build a grid that covers the tails -----------------------
+  grid_n   <- 2000                               # resolution (tweak if needed)
+  tail_pad <- 6 * max(sigmas)                    # same padding idea as before
+  x_grid   <- seq(min(x) - tail_pad,
+                  max(x) + tail_pad,
+                  length.out = grid_n)
+
+  cdf_grid <- cdf_fun(x_grid)
+
+  ## Remove duplicates (can happen in flat tails) ---------------------
+  ix <- !duplicated(cdf_grid)
+  cdf_u <- cdf_grid[ix]
+  x_u   <- x_grid[ix]
+
+  ## --- 2.  Monotone Hermite spline: probability -> quantile --------
+  q_spline <- splinefun(cdf_u, x_u, method = "monoH.FC")
+
+  ## --- 3.  Final inverse-CDF ---------------------------------------
   icdf_fun <- function(p) {
     stopifnot(all(p >= 0 & p <= 1))
-    vapply(p, function(pp) uniroot(function(q) cdf_fun(q) - pp, interval = c(lower, upper))$root, numeric(1))
+
+    ## exact edges
+    p[p == 0] <- min(cdf_u)
+    p[p == 1] <- max(cdf_u)
+
+    ## for probabilities outside the pre-computed range
+    p[p < min(cdf_u)] <- min(cdf_u)
+    p[p > max(cdf_u)] <- max(cdf_u)
+
+    q_spline(p)
   }
 
   sampler_fun <- function(n) {
